# Created by Q-ays.
# whosqays@gmail.com

# install elasticsearch_dsl before use

# elastic-search tools

from elasticsearch_dsl import Q, Search
from elasticsearch import Elasticsearch
from datetime import datetime


def inner_o2d_filter(o):
    for k in o:
        if isinstance(o[k], dict):
            inner_o2d_filter(o[k])
        if isinstance(o[k], datetime):
            o[k] = o[k].strftime("%Y-%m-%d %H:%M:%S")


def o2d(obj):
    is_arr = isinstance(obj, list)
    is_set = isinstance(obj, set)

    if is_set or is_arr:
        res = []
        for o in obj:
            if not isinstance(o, dict):
                d0 = o.to_dict(include_meta=True)
                d1 = d0.get('_source')
                d1['id'] = d0.get('_id')
                inner_o2d_filter(d1)
                res.append(d1)
            else:
                if o.get('_source'):
                    d0 = o.get('_source')
                    d0['id'] = o.get('_id')
                    inner_o2d_filter(d0)
                    res.append(d0)
                else:
                    res.append(o)

        return res

    if not isinstance(obj, dict):
        d0 = obj.to_dict(include_meta=True)
        d1 = d0.get('_source')
        d1['id'] = d0.get('_id')
        inner_o2d_filter(d1)
        return d1
    else:
        if obj.get('data'):
            data = obj.get('data')
            obj['data'] = o2d(data)

        return obj


doc_type0 = 'doc'


def repo_ref(doc_type):
    """
    基于es数据库增删改查的公共类
    :param doc_type:
    :return:
    """

    class BaseRepo:

        def __init__(self, ModelType, doc_type=doc_type):
            self.Model = ModelType
            self.index = ModelType.Index.name
            self.type = doc_type

        def add(self, data):
            print(data)
            model = self.Model(**data)
            model.save()
            return model

        def delete(self, did=None):
            if did:
                res = self.Model.get(did)
                res.delete()
                return res

        def update(self, did, data):
            model = self.Model.get(did)
            for key in data:
                if hasattr(model, key):
                    setattr(model, key, data[key])
            model.save()
            return model

        def get(self, did=None, p=None, **kwargs):
            """
            查询对象字段时，参数为对象
            :param did:
            :param p:
            :param kwargs:
            :return:
            """
            if did:
                if isinstance(did, list):
                    obj = self.Model.mget(did)
                else:
                    obj = self.Model.get(did)
            else:
                s = self.Model.search()
                for key in kwargs:
                    if isinstance(kwargs[key], dict):
                        s = s.query('nested', path=key, query=Q('match', **{str(key) + '.id': kwargs[key].get('id')}))
                    else:
                        s = s.query('match', **{key: kwargs[key]})

                if isinstance(p, list) or isinstance(p, tuple):
                    s = s[p[0]:p[1]]
                    res = s.execute()
                else:
                    res = s.scan()

                obj = []
                for o in res:
                    obj.append(o)

            return obj

        def paging(self, p=(0, 10), **kwargs):
            s = self.Model.search()
            for key in kwargs:
                if isinstance(kwargs[key], dict):
                    s = s.query('nested', path=key, query=Q('match', **{str(key) + '.id': kwargs[key].get('id')}))
                else:
                    s = s.query('match', **{key: kwargs[key]})

            if isinstance(p, list) or isinstance(p, tuple):
                s = s[p[0]:p[1]]
                res = s.execute()
                obj = []
                for o in res:
                    obj.append(o)

                data = dict()
                data['loc'] = p[0]
                data['size'] = p[1]
                data['total'] = res.hits.total
                data['data'] = obj

                return data

    return BaseRepo


class EsSearch:
    '''
     es查询其实是对repo_ref中get方法的完善（nested查询）
    '''

    def __init__(self, ES_HOST, http_auth=None):
        self.es = Elasticsearch([ES_HOST], http_auth=http_auth)

    def es_search(self, index, p=None, ns_key='id', **kwargs):
        s = Search(using=self.es, index=index)
        for key in kwargs:
            if isinstance(kwargs[key], dict):
                s = s.query('nested', path=key, query=Q('match', **{str(key) + '.' + ns_key: kwargs[key].get(ns_key)}))
            else:
                s = s.filter('term', **{key + '.keyword': kwargs[key]})

        if isinstance(p, list) or isinstance(p, tuple):
            s = s[p[0]:p[1]]
            res = s.execute()
        else:
            res = s.scan()

        return [{"id": o.meta.id, **o.to_dict()} for o in res]

    def paging(self, index: str, searches: dict, page=(0, 50), sort=None, types: str = None, ns_key='id', **kwargs):
        """

        :param index: index名称
        :param searches: 搜索字典
        :param page: (10,20) {"from": 10, "to": 20}
        :param sort: 排序
        :param types: 搜索类型
        :param ns_key:
        :param kwargs:
        :return:
        """

        s = Search(using=self.es, index=index)

        if searches:
            for key in searches:
                if isinstance(searches[key], dict):
                    s = s.query('nested', path=key,
                                query=Q('match', **{str(key) + '__' + ns_key: searches[key].get(ns_key)}))
                elif isinstance(searches[key], list):
                    s = s.query('range', **{key: {'gte': searches[key][0], 'lte': searches[key][1]}})
                else:
                    if types == 'term':
                        s = s.filter('term', **{key + '.keyword': searches[key]})
                    else:
                        s = s.filter('wildcard', **{key + '.keyword': '*' + searches[key] + '*'})
        if sort:
            s = s.sort(sort)  # 排序

        if isinstance(page, list) or isinstance(page, tuple):
            s = s[page[0]:page[1]]
            res = s.execute()

            obj = [{'id': o.meta.id, **o.to_dict()} for o in res]

            data = dict()
            data['loc'] = page[0]
            data['to'] = page[1]
            data['total'] = res.hits.total
            data['data'] = obj

            return data
        else:
            res = s.scan()

            return [{'id': o.meta.id, **o.to_dict()} for o in res]


Repo = repo_ref(doc_type0)



# Created by Q-ays.
# whosqays@gmail.com


from elasticsearch_dsl import Document, InnerDoc, Text, Keyword, Integer, Float, Boolean, Ip, Object, Nested, Date, \
    GeoPoint

from wisdoms.ms import rpc_wrapper
from wisdoms.es_db import inner_o2d_filter

from datetime import datetime


class EnumType(InnerDoc):
    name = Text(fields={'keyword': Keyword()})  # 枚举名称
    code = Text(fields={'keyword': Keyword()})  # 枚举code
    order = Integer()  # 序号


class Phone(Text):
    pass


class Email(Text):
    pass


class EnumObj(Object):
    pass


def enumObj():
    return EnumObj(EnumType)


def doc_spawn(MS_HOST):
    class BaseDocument(Document):
        user_ = Nested()
        org_ = Nested()
        addTime_ = Date()
        updTime_ = Date()
        status_ = Text()

        @classmethod
        def field2index_set(cls):
            try:
                cls.init()
            except Exception as e:
                print('es 模型初始化失败，模型字段类型可能冲突')
                print(e)

            try:

                index = cls.Index.name

                # todo 存表结构 return 表id
                res = rpc_wrapper('indexSetService', 'index_add', {'data': {'index': index, 'status': 'init'}},
                                  host_=MS_HOST)

                if res.get('code') == 1:
                    index_id = res.get('data').get('id')
                else:
                    print(res)
                    raise Exception('添加index表出错')

                datas = list()

                fields = cls._doc_type.mapping.properties._params.get('properties')

                i = 5
                for k in fields:
                    if k in ['user_', 'org_', 'status_', 'addTime_', 'updTime_']:
                        continue
                    data = dict()
                    data['index'] = index
                    data['index_id'] = index_id
                    data['field'] = k
                    data['sort'] = i
                    if isinstance(fields[k], EnumObj):
                        data['types'] = 'enum'
                        data['properties'] = ['code', 'value', 'name', 'label', 'order']
                    elif isinstance(fields[k], Phone):
                        data['name'] = '电话'
                        data['types'] = 'phone'
                    elif isinstance(fields[k], Email):
                        data['name'] = '邮箱'
                        data['types'] = 'email'
                    elif isinstance(fields[k], Date):
                        data['name'] = '日期'
                        data['types'] = 'date'
                    elif isinstance(fields[k], Integer):
                        data['types'] = 'int'
                    elif isinstance(fields[k], Ip):
                        data['types'] = 'ip'
                    elif isinstance(fields[k], Nested):
                        data['types'] = 'nested'
                    # 这行代码需要放到最后的 else if
                    elif isinstance(fields[k], Text):
                        data['types'] = 'str'
                    else:
                        pass

                    datas.append(data)
                    i = i + 2

                print(datas)

                res = rpc_wrapper('indexSetService', 'field_adds', {'data': datas}, host_=MS_HOST)
                print(res)

            except Exception as e:
                print('字段和表存入pg 失败')
                print(e)

        def dump(self, p=None):
            res = self.to_dict()

            inner_o2d_filter(res)

            d = dict()

            d['id'] = self.meta.id
            d['index'] = self.meta.index

            if p:
                for k in res:
                    if isinstance(res[k], dict):
                        d[k] = res[k].get(p, res[k])
                    else:
                        d[k] = res[k]

                return d
            else:
                return dict(d, **res)

        def save(self, *args, **kwargs):
            if not self.addTime_:
                self.addTime_ = datetime.now()

            return super().save(*args, **kwargs)

    return BaseDocument



import logging
import logging.handlers


def logs(file=None, mode='a', level=logging.DEBUG):

    logger = logging.getLogger(__name__)
    logger.setLevel(level=level)

    formatter = logging.Formatter('%(asctime)s - [line:%(lineno)d] %(filename)s - [%(levelname)s]: %(message)s')

    if file:
        handler = logging.FileHandler(file, mode)
        handler.setLevel(level)
        handler.setFormatter(formatter)
        logger.addHandler(handler)
    else:
        handler = logging.StreamHandler()
        handler.setLevel(level)
        handler.setFormatter(formatter)
        logger.addHandler(handler)

    return logger


def pro_logs(mode='a', level=logging.DEBUG):
    logger = logging.getLogger(__name__)
    logger.setLevel(level=level)

    formatter = logging.Formatter('%(asctime)s - [line:%(lineno)d] %(filename)s - [%(levelname)s]: %(message)s')

    handler = logging.handlers.HTTPHandler('localhost:5000', '/api/v1.0/logger', method='POST')
    handler.setLevel(level)
    handler.setFormatter(formatter)

    logger.addHandler(handler)

    return logger

if __name__ == '__main__':
    # logtest = logs('./logtest.log', 'w')
    # logtest.debug('fdsafasfsa')
    # logtest.info('abcadfa')
    # logtest.warning('ifjdsiafpasfapfdj')
    # logtest.error('fdsiafjasofjasdifojas')
    #
    # logtest2 = logs()
    # logtest2.debug('fadsfasfas')
    # logtest2.info('fsdiafjoasfoisdf')
    # logtest2.warning('fjsiadfjoasdjfiosa')
    # logtest2.error('ifjdsoafisadfjfjaisdfos')
    # try:
    #     a = 1 / 0
    # except Exception as e:
    #     logtest2.error(e)

    logtest3 = pro_logs()
    logtest3.info('adfadsf')
    logtest3.warning({'dfijdsa':'dfidsajo'})
    logtest3.error('adfadsf')
    logtest3.debug('adfadsf')